if (!window.dojo) alert('PlaceholderValues.js requires dojo.js');

function PlaceholderValues(){}

/**
 * Determines if all the placeholders in window.nameToPseudonymMap
 * have a value.
 * @return true if the user has filled out a value for all placeholders.
 */
PlaceholderValues.areAvailable = function()
{
	if (!window.nameToPseudonymMap) return false;
	// ensure that there is a value for every placeholder
	for (var placeholder in window.nameToPseudonymMap)
	{
		var pseudo = window.nameToPseudonymMap[placeholder];

		// update the model
		PlaceholderValues._flushTextValue(pseudo);
		if (!CtrlUtils.hasValue(pseudo))
		{
			return false;
		}
	}

	return true;
};

/**
 * Since the data model is only updated for text value "onblur", it's possible
 * in IE that the user can enter a value, click 'Run', and get stale data if
 * the blur didn't fire (which it doesn't always do when selecting things across frames).
 * This flushes the current visible value so that the data model is correct again.
 * This is only needed for text attributes, but doesn't hurt if called for others.
 */
PlaceholderValues._flushTextValue = function(attributeName)
{
	var flushFunction = window[attributeName + '_ChangeText'];
	if (flushFunction) flushFunction();
};

/**
 * Returns all placeholder values as XML.
 * Assume values can be retrieved from their "XXX_Value()" functions.
 * All placeholders must have an entry in window.nameToPseudonymMap.
 *
 * @returns seomthing like
 * <placeholder-values><placeholder><name>%x</name><value>Foo</value></placeholder></placeholder-values>
 */
PlaceholderValues.asXml = function()
{
	var doc = dojox.data.dom.createDocument();
		
	var placeholderValues = doc.createElement('placeholder-values');
	doc.appendChild(placeholderValues);

	for (var placeholderName in window.nameToPseudonymMap)
	{
		var pseudo = window.nameToPseudonymMap[placeholderName];
		
		var placeholder = doc.createElement('placeholder');
		placeholderValues.appendChild(placeholder);
		
		var name = doc.createElement('name');
		name.appendChild(
			doc.createTextNode(placeholderName));
		placeholder.appendChild(name);
		
		// update the model
		PlaceholderValues._flushTextValue(pseudo);

		var value = doc.createElement('value');
		var getValue = CtrlUtils.getValue(pseudo);
		value.appendChild(
			doc.createTextNode(getValue));
		placeholder.appendChild(value);
	}

	return dojox.data.dom.innerXML(doc);
};